@@ -1,6 +1,6 @@
 package LWP::EventLoop;
 
-# $Id: EventLoop.pm,v 1.10 1998/03/17 11:06:37 aas Exp $
+# $Id: EventLoop.pm,v 1.14 1998/04/20 13:15:44 aas Exp $
 
 # Copyright 1997-1998 Gisle Aas.
 #
@@ -9,7 +9,8 @@ package LWP::EventLoop;
 
 
 use strict;
-use vars qw($DEBUG);
+use vars qw($DEBUG $VERSION);
+$VERSION = "0.11";
 
 # If Time::HiRes is installed then we can get better timeout resolution
 BEGIN {
@@ -17,7 +18,7 @@ BEGIN {
 	require Time::HiRes;
 	Time::HiRes->import('time');
     };
-    print STDERR $@ if $@ && $DEBUG;
+    warn $@ if $@ && $DEBUG;
 }
 
 my $atid = 0;  # incremented in order to generate unique after/at identifiers
@@ -171,7 +172,7 @@ sub _vec
     }
     $self->{$cachebits} = $vec;
     if (@closed) {
-	print "1) Getting rid of closed handles: @closed\n";
+	warn "Getting rid of closed handles: @closed" if $DEBUG;
 	$self->forget(@closed);
     }
 }
@@ -182,7 +183,7 @@ sub _check_closed
     my @closed = grep {!defined fileno($_)}
                     map { $_->[0] } values %{$self->{fh}};
     if (@closed) {
-	print "2) Getting rid of closed handles: @closed\n";
+	warn "Getting rid of closed handles: @closed" if $DEBUG;
 	$self->forget(@closed);
     }
 }
@@ -196,46 +197,6 @@ sub _fh
     delete $self->{fh}{int($fh)} unless grep defined,  @callbacks;
 }
 
-sub dump
-{
-    my $self = shift;
-    print "$self\n";
-    my $now = time();
-    for (values %{$self->{fh}}) {
-	my($fh,$r,$w,$e,$timeout, $pending) = @$_;
-	printf "  %x %-17s %2d", int($fh), ref($fh), fileno($fh);
-	for ($r, $w, $e) {
-	    if (defined) {
-		if (ref($_) eq "CODE") {
-		    printf "  %-10s", "CODE";
-		} else {
-		    printf "  %-10s", $_;
-		}
-	    } else {
-		printf "  %-10s", "-";
-	    }
-	}
-	if ($timeout) {
-	    my @t = @$timeout;
-	    $t[2] = $now - $t[2];
-	    print "[@t]";
-	}
-	if ($pending) {
-	    print "<@$pending>";
-	}
-	print "\n";
-    }
-    for ("_r", "_w", "_e") {
-	print "  $_: ";
-	if (defined $self->{$_}) {
-	    print unpack("b*", $self->{$_});
-	} else {
-	    print "undef";
-	}
-	print "\n";
-    }
-}
-
 sub empty
 {
     my $self = shift;
@@ -245,19 +206,18 @@ sub empty
 sub one_event   # or none
 {
     my $self = shift;
-
+    my $timeout = shift;
     my $now = time;
-    my $timeout = 60;
     my $at = $self->{'at'};
     if (@$at) {
-	$timeout = $at->[0][1] - $now;
-	if ($timeout <= 0) {
+	my $at_timeout = $at->[0][1] - $now;
+	if ($at_timeout <= 0) {
 	    # the first timer has expired
 	    my($id, $time, $cb) = @{shift @$at};
 	    if ($DEBUG) {
 		print STDERR "timer callback $id";
-		if ($timeout < -0.01) {
-		    printf STDERR " (%.2fs late)\n", -$timeout;
+		if ($at_timeout < -0.01) {
+		    printf STDERR " (%.2fs late)\n", -$at_timeout;
 		} else {
 		    print STDERR "\n";
 		}
@@ -266,6 +226,7 @@ sub one_event   # or none
 	    warn $@ if $@ && $^W;
 	    return;
 	}
+	$timeout = $at_timeout if !$timeout || $at_timeout < $timeout;
     }
 
     for (values %{$self->{fh}}) {
@@ -287,7 +248,7 @@ sub one_event   # or none
 	    $timeout_spec->[2] = $now;  # record activity
 	    return;
 	}
-	$timeout = $timeout_sec if $timeout_sec < $timeout;
+	$timeout = $timeout_sec if !$timeout || $timeout_sec < $timeout;
     }
     if ($DEBUG) {
 	print STDERR "select(";
@@ -336,7 +297,7 @@ sub one_event   # or none
 	    }
 	}
 	if (@closed) {
-	    print "3) Getting rid of closed handles: @closed\n";
+	    warn "Getting rid of closed handles: @closed" if $DEBUG;
 	    $self->forget(@closed);
 	}
     } else {
@@ -366,7 +327,184 @@ sub _fh_callback
 sub run
 {
     my $self = shift;
-    $self->one_event until $self->empty;
+    my $done;
+    if (my $timeout = shift) {
+	$self->after($timeout, sub {$done++});
+    };
+    $self->one_event until $self->empty || $done;
+}
+
+sub dump  # for debugging
+{
+    my $self = shift;
+    print "$self\n";
+    my $now = time();
+    for (values %{$self->{fh}}) {
+	my($fh,$r,$w,$e,$timeout, $pending) = @$_;
+	printf "  %x %-17s %2d", int($fh), ref($fh), fileno($fh);
+	for ($r, $w, $e) {
+	    if (defined) {
+		if (ref($_) eq "CODE") {
+		    printf "  %-10s", "CODE";
+		} else {
+		    printf "  %-10s", $_;
+		}
+	    } else {
+		printf "  %-10s", "-";
+	    }
+	}
+	if ($timeout) {
+	    my @t = @$timeout;
+	    $t[2] = $now - $t[2];
+	    print "[@t]";
+	}
+	if ($pending) {
+	    print "<@$pending>";
+	}
+	print "\n";
+    }
+    for ("_r", "_w", "_e") {
+	print "  $_: ";
+	if (defined $self->{$_}) {
+	    print unpack("b*", $self->{$_});
+	} else {
+	    print "undef";
+	}
+	print "\n";
+    }
+    my @at = @{$self->{"at"}};
+    if (@at) {
+	print "  at:";
+	for (@at) {
+	    my($id,$time,$cb) = @$_;
+	    $time = sprintf("%.3g", $time - $now);
+	    print " $id/${time}s/$cb";
+	}
+	print "\n";
+    }
 }
 
 1;
+
+__END__
+
+=head1 NAME
+
+LWP::EventLoop - Watch file descriptors and timers
+
+=head1 SYNOPSIS
+
+ use LWP::EventLoop;
+ $mainloop = LWP::EventLoop->new;
+ $mainloop->readable(\*STDIN, sub {sysread(STDIN, $buf, 100)});
+ $mainloop->after(10, sub { print "10 sec later"} );
+ $mainloop->run;
+
+=head1 DESCRIPTION
+
+The I<LWP::EventLoop> class define objects that can watch file
+descriptors and timers and will invoke callback methods when events on
+these happens.  Usually you will only have a single instance of this
+class in any application.  The I<LWP::MainLoop> module creates a
+single instance and provide an interface to it.  The I<LWP::EventLoop> is
+really just a wrapping of the select() function.
+
+The following methods are provided:
+
+=over 4
+
+=item $e = LWP::EventLoop->new
+
+The constructor takes now arguments.
+
+=item $e->readable($io, [$callback])
+
+Register the specified IO handle as being monitored for readable
+status.  When the handle becomes readable the specified callback will
+be invoked.  The handle can be unregistered by giving an C<undef>
+argument as the $callback.
+
+Callbacks can either by an CODE reference (which is called with the
+handle as argument) or they can be a plain scalar strings which are
+taken to be method names that are called on the given handle object.
+The callback can also be an array reference.  The first element of the
+array must be a CODE reference or a method name.  The rest is taken to
+be additional arguments passed during callback invocation.
+ 
+The default callback is to invoke the $io->readable method.
+
+
+=item $e->writable($io, [$callback])
+
+Like $e->readable, but watch the handle for writable status.  The
+default callback to invoke is the $io->writable method.
+
+=item $e->timeout($io, $secs, [$callback])
+
+Register a callback to be invoked if nothing happens on the given IO
+handle for some number of seconds.  Callbacks take the same form as
+for $e->readable.  The default callback is to invoke the $io->inactive
+method.  Pass 0 as the $secs argument to disable timeout for this
+handle.
+
+=item $e->after($secs, $timer_callback)
+
+Set up a callback to be invoked after the given number of seconds.
+The callback must be a CODE reference.  This method returns an
+identifier that can be used to cancel this timer using the
+$e->forget method.
+
+=item $e->at($time, $timer_callback)
+
+Set up a callback to be invoked at the given time.  The $e->after is
+really the same as $e->at(time + $secs, $timer_callback).
+
+=item $e->forget($io_timer,...)
+
+Unregister all callbacks for the given IO handles and timers.  One or
+more arguments can be given.  Each argument can either be an IO handle
+reference or an identifier as returned by $e->after or $e->at.
+
+=item $e->forget_all
+
+Unregister all callbacks.  The state of the LWP::EventLoop will be as
+after construction.
+
+=item $e->empty
+
+Return TRUE if no timer callbacks or IO handles to watch are
+registered.
+
+=item $e->one_event( [$timeout] )
+
+Wait for a single event to happen (but no longer than $timeout
+seconds) and call the corresponding callback routine.  Can also return
+without calling any callback routine.
+
+=item $e->run( [$timeout] )
+
+Call $e->one_event until either all timer callbacks and IO handles are
+gone or until the specified number of seconds has elapsed.
+
+=item $e->dump
+
+Will print the state of the I<LWP::EventLoop> object to the currently
+selected file handle.  Mainly useful for debugging.  Setting the
+$LWP::EventLoop::DEBUG variable to an TRUE value can also be useful
+while debugging.
+
+=back
+
+
+=head1 SEE ALSO
+
+L<LWP::MainLoop>
+
+=head1 COPYRIGHT
+
+Copyright 1997-1998, Gisle Aas
+
+This library is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself.
+
+=cut
